@@ -4,16 +4,18 @@
 
 import csv
 import uuid
+import json
+import warnings
 from abc import ABCMeta, abstractmethod
 from enum import Enum
 from collections import defaultdict
 from pathlib import Path
-from typing import Optional, List, Dict, Any, Iterable, Union, Callable, Set, IO
+import typing as tp
 
 from .render import make_experiment_standalone_page, html_inlinize
 
 
-DisplayableType = Union[bool, int, float, str]
+DisplayableType = tp.Union[bool, int, float, str]
 
 
 class ExperimentValidationError(Exception):
@@ -33,7 +35,7 @@ class _DictSerializable:
     All classes that are transmitted to Javascript must subclass this
     """
 
-    def _asdict(self) -> Dict[str, Any]:
+    def _asdict(self) -> tp.Dict[str, tp.Any]:
         return self.__dict__
 
 
@@ -58,7 +60,7 @@ class Displays:
     DISTRIBUTION = 'DISTRIBUTION'               #: Distribution plot data
 
 
-def validate_colormap(cm: Optional[str]) -> None:
+def validate_colormap(cm: tp.Optional[str]) -> None:
     # We don't want `d3.interpolateTurbo` but just `interpolateTurbo`
     if cm is not None and not cm.startswith("interpolate") and not cm.startswith("scheme"):
         raise ExperimentValidationError(f"""Invalid colormap `{cm}`.
@@ -67,6 +69,82 @@ Examples include `interpolateSpectral`, `interpolateViridis`, `interpolateSinebo
 """)
 
 
+class _StreamlitHelpers:
+    component: tp.Optional[tp.Callable[..., tp.Any]] = None
+
+    @staticmethod
+    def is_running_within_streamlit() -> bool:
+        try:
+            import streamlit as st
+        except:  # pylint: disable=bare-except
+            return False
+        return bool(st._is_running_with_streamlit)
+
+    @classmethod
+    def create_component(cls) -> None:
+        if cls.component is not None:
+            return
+        import streamlit as st
+        try:
+            import streamlit.components.v1 as components
+        except ModuleNotFoundError as e:
+            raise RuntimeError(f'Your streamlit version ({st.__version__}) is too old and does not support components. Please update streamlit with `pip install -U streamlit`') from e
+        assert st._is_running_with_streamlit
+
+        built_path = (Path(__file__).parent / "static" / "built" / "streamlit_component").resolve()
+        assert (built_path / "index.html").is_file(), f"""HiPlot component does not appear to exist in {built_path}
+If you did not install hiplot using official channels (pip, conda...), maybe you forgot to build javascript files?
+See https://facebookresearch.github.io/hiplot/contributing.html#building-javascript-bundle
+"""
+        cls.component = components.declare_component("hiplot", path=str(built_path))
+
+    @classmethod
+    def create_instance_wrapper(
+            cls,
+            exp: "Experiment",
+            ret: tp.Union[str, tp.List[str], None] = None,
+            key: tp.Optional[str] = None
+    ) -> tp.Any:
+        cls.create_component()
+        possible_returns = ['selected_uids', 'filtered_uids', 'brush_extents']
+        if key is None:
+            warnings.warn(r"""Creating a HiPlot component with key=None will make refreshes slower.
+Please use `experiment.display_st(..., key=\"some_unique_key\")`""")
+        ret_type_for_js = ret if isinstance(ret, list) else []
+        if isinstance(ret, str):
+            ret_type_for_js = [ret]
+        for r in ret_type_for_js:
+            assert r in possible_returns, f"Unknown return type {r}. Possible values: {','.join(possible_returns)}"
+        assert cls.component is not None
+        js_ret = cls.component(experiment=json.dumps(exp._asdict()), ret=ret_type_for_js, key=key)  # pylint: disable=not-callable
+
+        if js_ret is None:
+            js_ret = [None] * len(ret_type_for_js)
+
+        for idx, r in enumerate(ret_type_for_js):
+            if js_ret[idx] is not None:
+                continue
+            # Use default value
+            if r in ['selected_uids', 'filtered_uids']:
+                js_ret[idx] = [dp.uid for dp in exp.datapoints]
+            if r == 'brush_extents':
+                js_ret[idx] = []
+
+        if ret is None:
+            return None
+        if isinstance(ret, str):
+            return js_ret[0]
+        return js_ret
+
+
+def _is_running_ipython() -> bool:
+    try:
+        get_ipython()  # type: ignore
+        return True
+    except NameError:
+        return False
+
+
 class ValueDef(_DictSerializable):
     """
     Provides a custom type, color, etc.. for a column.
@@ -82,17 +160,17 @@ class ValueDef(_DictSerializable):
 
     def __init__(
             self,
-            value_type: Optional[ValueType] = None,
-            colors: Optional[Dict[Any, str]] = None,
-            colormap: Optional[str] = None,
-            label_css: Optional[str] = None
+            value_type: tp.Optional[ValueType] = None,
+            colors: tp.Optional[tp.Dict[tp.Any, str]] = None,
+            colormap: tp.Optional[str] = None,
+            label_css: tp.Optional[str] = None
     ) -> None:
         self.type = value_type
         self.colors = colors
         self.colormap = colormap
         self.label_css = label_css
-        self.force_value_min: Optional[float] = None
-        self.force_value_max: Optional[float] = None
+        self.force_value_min: tp.Optional[float] = None
+        self.force_value_max: tp.Optional[float] = None
 
     def force_range(self, minimum: float, maximum: float) -> "ValueDef":
         """
@@ -111,7 +189,7 @@ class ValueDef(_DictSerializable):
                     )
         validate_colormap(self.colormap)
 
-    def _asdict(self) -> Dict[str, Any]:
+    def _asdict(self) -> tp.Dict[str, tp.Any]:
         return {
             "type": self.type.value if self.type is not None else None,
             "colors": self.colors,
@@ -130,7 +208,7 @@ class Datapoint(_DictSerializable):
 
     :ivar uid: A unique identifier for this datapoint
     :ivar values: A dictionnary with arbitrary metrics/values
-    :ivar from_uid: The uid of the parent :class:`Datapoint` (optional)
+    :ivar from_uid: The uid of the parent :class:`Datapoint` (tp.Optional)
 
     :Example:
 
@@ -145,7 +223,7 @@ class Datapoint(_DictSerializable):
         hip.Experiment(datapoints=[dp1, dp2]).display()  # Render in an ipython notebook
     """
 
-    def __init__(self, values: Dict[str, DisplayableType], *, uid: Optional[str] = None, from_uid: Optional[str] = None) -> None:
+    def __init__(self, values: tp.Dict[str, DisplayableType], *, uid: tp.Optional[str] = None, from_uid: tp.Optional[str] = None) -> None:
         self.uid = uid if uid is not None else str(uuid.uuid4())
         self.values = values
         self.from_uid = from_uid
@@ -180,26 +258,26 @@ class Experiment(_DictSerializable):
     """
 
     def __init__(self,
-                 datapoints: Optional[List[Datapoint]] = None,
-                 parameters_definition: Optional[Dict[str, ValueDef]] = None,
-                 colormap: Optional[str] = None,
+                 datapoints: tp.Optional[tp.List[Datapoint]] = None,
+                 parameters_definition: tp.Optional[tp.Dict[str, ValueDef]] = None,
+                 colormap: tp.Optional[str] = None,
                  ) -> None:
         self.datapoints = datapoints if datapoints is not None else []
         self.parameters_definition = parameters_definition if parameters_definition is not None else defaultdict(ValueDef)
         self.colormap = colormap if colormap is not None else "interpolateTurbo"
-        self.colorby: Optional[str] = None
-        self._displays: Dict[str, Dict[str, Any]] = {}
+        self.colorby: tp.Optional[str] = None
+        self._display_data: tp.Dict[str, tp.Dict[str, tp.Any]] = {}
 
     def validate(self) -> "Experiment":
         """
         Makes sure that this object is valid. Raises a :class:`hiplot.ExperimentValidationError` otherwise.
         Experiments with circular references, non-existent parents, or without datapoints are invalid.
         """
-        seen: Set[str] = set()
-        dp_lookup: Dict[str, Datapoint] = {dp.uid: dp for dp in self.datapoints}
+        seen: tp.Set[str] = set()
+        dp_lookup: tp.Dict[str, Datapoint] = {dp.uid: dp for dp in self.datapoints}
         for p in self.datapoints:
             if p.uid not in seen:
-                seen_now: Set[str] = {p.uid}
+                seen_now: tp.Set[str] = {p.uid}
                 dp = p
                 while dp.from_uid is not None and dp.from_uid not in seen:
                     if dp.from_uid in seen_now:
@@ -215,24 +293,70 @@ class Experiment(_DictSerializable):
         validate_colormap(self.colormap)
         return self
 
-    def display(self, force_full_width: bool = False, store_state_key: Optional[str] = None) -> "ExperimentDisplayed":
+    def display(self, force_full_width: bool = False, store_state_key: tp.Optional[str] = None, **kwargs: tp.Any) -> "ExperimentDisplayed":
         """
         Displays an experiment in an ipython notebook.
 
         :param force_full_width: allows to force to have 100% width on Jupyter Notebooks only.
         :param store_state_key: a string identifier for the HiPlot instance.
-            If not `None`, HiPlot will store dynamic modifications (removing/reordering columns...)
-            in the URL, and restore them when calling `display` with the same value for `store_state_key` - see :ref:`tutoNotebookState`
+            If not ``None``, HiPlot will store dynamic modifications (removing/reordering columns...)
+            in the URL, and restore them when calling ``display`` with the same value for ``store_state_key`` - see :ref:`tutoNotebookState`
         :returns: An :class:`ExperimentDisplayed` object that can be used to interact with the visualization
             - only implemented for Jupyter notebook.
             See :ref:`tutonotebookdisplayedexperiment`
         """
+        if not _is_running_ipython():
+            if _StreamlitHelpers.is_running_within_streamlit():
+                raise RuntimeError(r"""`experiment.display` can only be called with ipython.
+It appears that you are trying to create a HiPlot visualization in Streamlit: you should use `display_st`""")
+            raise RuntimeError(r"""`display` can only be called on an ipython context. Are you in a notebook?
+- To render an experiment to HTML, use `experiment.to_html(file_name)` or `html_page = experiment.to_html()`
+- To render an experiment in Streamlit, use `experiment.display_st`""")
         from .ipython import display_exp  # pylint: disable=cyclic-import
-
         self.validate()
-        return display_exp(self, force_full_width=force_full_width, store_state_url=store_state_key)
+        return display_exp(self, force_full_width=force_full_width, store_state_url=store_state_key, **kwargs)
 
-    def to_html(self, file: Optional[Union[Path, str, IO[str]]] = None) -> str:
+    # pylint: disable=function-redefined
+    @tp.overload
+    def display_st(self, *, ret: str, key: tp.Optional[str] = None) -> tp.Any:
+        pass
+
+    @tp.overload
+    def display_st(self, *, ret: tp.List[str], key: tp.Optional[str] = None) -> tp.List[tp.Any]:
+        pass
+
+    @tp.overload
+    def display_st(self, *, key: tp.Optional[str] = None) -> None:
+        pass
+
+    def display_st(self, *, ret: tp.Union[str, tp.List[str], None] = None, key: tp.Optional[str] = None) -> tp.Any:
+        """
+        Displays an experiment in a Streamlit app - see :ref:`tutoStreamlit`
+
+        :param key: Unique key for the streamlit component. It is strongly recommended to give some unique string.
+        :param ret: Specify what HiPlot should return.
+        :returns: Return value depends on ``ret``
+
+        :Example:
+
+        .. code-block:: python
+
+            exp.display_st(key="hiplot1")
+            brush_extents = exp.display_st(key="hiplot2", ret="brush_extents")
+            brush_extents, selected_uids = exp.display_st(key="hiplot3", ret=["brush_extents", "selected_uids"])
+
+        """
+        if not _StreamlitHelpers.is_running_within_streamlit():
+            if _is_running_ipython():
+                raise RuntimeError(r"""`experiment.display_st` can only be called in a streamlit script.
+It appears that you are trying to create a HiPlot visualization in ipython: you should use `display` instead of `display_st`""")
+            raise RuntimeError(r"""`experiment.display_st` can only be called in a streamlit script.
+To render an experiment to HTML, use `experiment.to_html(file_name)` or `html_page = experiment.to_html()`""")
+        return _StreamlitHelpers.create_instance_wrapper(exp=self, ret=ret, key=key)
+
+    # pylint: enable=function-redefined
+
+    def to_html(self, file: tp.Optional[tp.Union[Path, str, tp.IO[str]]] = None, **kwargs: tp.Any) -> str:
         """
         Returns the content of a standalone .html file that displays this experiment
         without any dependency to HiPlot server or static files.
@@ -242,6 +366,7 @@ class Experiment(_DictSerializable):
         """
         self.validate()
         html = make_experiment_standalone_page(options={
+            **kwargs,
             'experiment': self._asdict()
         })
         html = html_inlinize(html)
@@ -252,7 +377,7 @@ class Experiment(_DictSerializable):
                 file.write(html)
         return html
 
-    def to_csv(self, file: Union[Path, str, IO[str]]) -> None:
+    def to_csv(self, file: tp.Union[Path, str, tp.IO[str]]) -> None:
         """
         Dumps this Experiment as a .csv file.
         Information about display_data, parameters definition will be lost.
@@ -265,8 +390,8 @@ class Experiment(_DictSerializable):
         else:
             return self._to_csv(file)
 
-    def _to_csv(self, fh: IO[str]) -> None:
-        fieldnames: Set[str] = set()
+    def _to_csv(self, fh: tp.IO[str]) -> None:
+        fieldnames: tp.Set[str] = set()
         for dp in self.datapoints:
             for f in dp.values.keys():
                 fieldnames.add(f)
@@ -279,26 +404,26 @@ class Experiment(_DictSerializable):
                 "from_uid": dp.from_uid,
             })
 
-    def _asdict(self) -> Dict[str, Any]:
+    def _asdict(self) -> tp.Dict[str, tp.Any]:
         return {
             "datapoints": [d._asdict() for d in self.datapoints],
             "parameters_definition": {k: v._asdict() for k, v in self.parameters_definition.items()},
             "colormap": self.colormap,
             "colorby": self.colorby,
-            "_displays": self._displays,
+            "display_data": self._display_data,
         }
 
     def remove_missing_parents(self) -> "Experiment":
         """
         Sets :attr:`hiplot.Datapoint.from_uid` to None when set to a non-existing Datapoint.
         """
-        existing_dp: Set[str] = set((dp.uid for dp in self.datapoints))
+        existing_dp: tp.Set[str] = set((dp.uid for dp in self.datapoints))
         for dp in self.datapoints:
             if dp.from_uid not in existing_dp:
                 dp.from_uid = None
         return self
 
-    def display_data(self, plugin: str) -> Dict[str, Any]:
+    def display_data(self, plugin: str) -> tp.Dict[str, tp.Any]:
         """
         Retrieve data dictionnary for a plugin, which can be modified.
 
@@ -314,10 +439,10 @@ class Experiment(_DictSerializable):
             })
 
         """
-        return self._displays.setdefault(plugin, {})
+        return self._display_data.setdefault(plugin, {})
 
     @staticmethod
-    def from_iterable(it: Iterable[Dict[str, Any]]) -> "Experiment":
+    def from_iterable(it: tp.Iterable[tp.Dict[str, tp.Any]]) -> "Experiment":
         """
         Creates a HiPlot experiment from an iterable/list of dictionnaries.
         This is the easiest way to generate an `hiplot.Experiment` object.
@@ -341,7 +466,7 @@ class Experiment(_DictSerializable):
         )
 
     @staticmethod
-    def from_csv(file: Union[Path, str, IO[str]]) -> "Experiment":
+    def from_csv(file: tp.Union[Path, str, tp.IO[str]]) -> "Experiment":
         """
         Creates a HiPlot experiment from a CSV file.
 
@@ -354,7 +479,7 @@ class Experiment(_DictSerializable):
             return Experiment.from_iterable(csv.DictReader(file))
 
     @staticmethod
-    def from_dataframe(dataframe: Any) -> "Experiment":  # No type hint to avoid having pandas as an additional dependency
+    def from_dataframe(dataframe: tp.Any) -> "Experiment":  # No type hint to avoid having pandas as an additional dependency
         """
         Creates a HiPlot experiment from a pandas DataFrame.
 
@@ -363,7 +488,7 @@ class Experiment(_DictSerializable):
         return Experiment.from_iterable(dataframe.to_dict(orient='records'))
 
     @staticmethod
-    def merge(xp_dict: Dict[str, "Experiment"]) -> "Experiment":
+    def merge(xp_dict: tp.Dict[str, "Experiment"]) -> "Experiment":
         """
         Merge several experiments into a single one
         """
@@ -379,6 +504,8 @@ class Experiment(_DictSerializable):
             ]
             if subxp.parameters_definition is not None:
                 xp.parameters_definition.update(subxp.parameters_definition)
+            for d, v in subxp._display_data.items():
+                xp.display_data(d).update(v)
         return xp
 
 
@@ -386,7 +513,7 @@ class ExperimentFetcherDoesntApply(Exception):
     pass
 
 
-ExperimentFetcher = Callable[[str], Experiment]
+ExperimentFetcher = tp.Callable[[str], Experiment]
 
 
 class ExperimentDisplayed(metaclass=ABCMeta):
@@ -395,13 +522,13 @@ class ExperimentDisplayed(metaclass=ABCMeta):
     Read more in :ref:`tutoNotebookDisplayedExperiment`
     """
     @abstractmethod
-    def get_selected(self) -> List[Datapoint]:
+    def get_selected(self) -> tp.List[Datapoint]:
         """
         Returns a list of currently rendered datapoints in the parallel plot
         """
 
     @abstractmethod
-    def get_brush_extents(self) -> Dict[str, Dict[str, Any]]:
+    def get_brush_extents(self) -> tp.Dict[str, tp.Dict[str, tp.Any]]:
         """
         Returns a dictionnary, where keys corresponds to columns currently brushed in parallel plot,
         and values contain information about the current brush.
